/*
  main.c 
*/

#include "mcc_generated_files/system.h"
#include "mcc_generated_files/tmr1.h"
#include "mcc_generated_files/ext_int.h"
#include "i2c_1.h"
#include <xc.h>
#include "AQM1602.h"
#include "DHT20.h"
#include "ADT7410.h"
#include "string.h"
#define FCY      _XTAL_FREQ
#include <libpic30.h> 

#define     LED_Pilot       LATAbits.LATA0
#define     LED_Heat        LATAbits.LATA1
#define     Heat_Power      LATAbits.LATA2
#define     St_Btn          PORTBbits.RB2

#define     w5_sec          0x9896
#define     w500ms          0xF41

#define     ADT7410_addr    0x49

#define     NUM_L0          PORTBbits.RB6
#define     NUM_L2          PORTBbits.RB7
#define     NUM_L4          PORTBbits.RB10
#define     NUM_L8          PORTBbits.RB11

#define     NUM_H0          PORTBbits.RB12
#define     NUM_H2          PORTBbits.RB13
#define     NUM_H4          PORTBbits.RB14
#define     NUM_H8          PORTBbits.RB15

int get_target_temp(void);

int int_flg = 0;

void myTimerISR(void)
{
    int_flg = 2;
}
 
int main(void)
{
    char c_buf[30];
    int target_temp, state_flg, tm_cnt, past_time;
    float ht_temp, heat_tmp, h_data[2];

    int a;
    
    // initialize the device
    SYSTEM_Initialize();

    InitI2C_Master();
    init_LCD();
    
    TMR1_SetInterruptHandler (myTimerISR);

    state_flg = 0;

    while (1)
    {
        TMR1 = 0x00;
        TMR1_Start();
        int_flg = 0;
        while(!int_flg) ;
        
        switch(int_flg)
        {
            case 1: // Start or Stop
                    a = 0; 
                    if(!state_flg)
                    {
                        a = 1;
                        target_temp = get_target_temp();
                        heat_tmp = target_temp * 1.92 - 24.14;
                        tm_cnt = past_time = 0;
                    }

                    LED_Pilot = LED_Heat = Heat_Power = a;

                    state_flg = !state_flg;
                    
                    for(a = 0; a< 5; a++)
                    {
                        while(!St_Btn) ;
                        __delay_ms(10);
                    }
                    
                    PR1 = w500ms;
                    break;
                    
            case 2: // Controll Heater
                    LCD_Cls();
                    if(state_flg)
                    {
                        ADT7410_get_temp(ADT7410_addr, &ht_temp);
                        DHT20_Get_Data(h_data);
                        a = 1;
                        if(ht_temp > heat_tmp)
                        {
                            if(past_time < 10)
                            {
                                if( h_data[1] > (target_temp * 0.67)) a = 0; 
                            }
                            else
                            {
                                if( h_data[1] > (target_temp * 0.95)) a = 0; 
                            }                            
                        }
                        LED_Heat = Heat_Power = a;
            
                        tm_cnt ++;
                        if(tm_cnt == 12) {tm_cnt = 0; past_time ++;}
                        
                        DHT20_Get_Data(h_data);

                        sprintf(c_buf,"T:%0.1f'c", h_data[1]);
                        LCD_Print_L(0,0,c_buf);
    
                        sprintf(c_buf,"H:%0.1f", h_data[0]);
                        LCD_Print_L(0,1,c_buf);
                        LCD_Print("% ");
                        sprintf(c_buf,"N:%d", past_time);
                        LCD_Print(c_buf);
                        PR1 = w5_sec;
                    }
                    else
                    {
                        a = get_target_temp();
                        sprintf(c_buf,"Target:%d'c",a);
                        LCD_Print_L(0,0,c_buf);
                        PR1 = w500ms;
                    }
                    break;
        }
    }

    return 1;
}

int get_target_temp()
{
  int a;

    a = 0; 
    if(NUM_H0) a = 1;
    if(NUM_H2) a += 2;
    if(NUM_H4) a += 4;
    if(NUM_H8) a += 8;
    a *= 10;

    if(NUM_L0) a += 1;
    if(NUM_L2) a += 2;
    if(NUM_L4) a += 4;
    if(NUM_L8) a += 8;
    
    return a;
}

/**
 End of File
*/


